#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# wls_install::fmwlogdir
#
# @summary This defined type allows you change a log folder location of a FMW server.
# Here is an example on how you could use this:
# 
# ```puppet
# wls_install::fmwlogdir{'AdminServer':
#   middleware_home_dir    => "/opt/oracle/middleware11gR1",
#   weblogic_user          => "weblogic",
#   weblogic_password      => "weblogic1",
#   os_user                => "oracle",
#   os_group               => "dba",
#   download_dir           => "/data/install"
#   log_dir                => "/var/log/weblogic"
#   adminserver_address    => "localhost",
#   adminserver_port       => 7001,
#   server                 => "AdminServer",
# }
# ```
# ```
#
# @param [Stdlib::Absolutepath] middleware_home_dir
#    Middleware home directory.
#
# @param [String[1]] adminserver_address
#    The address of the admin server.
#    May be a DNS name or a physical IP address.
#
# @param [Integer] adminserver_port
#    The IP port the admin server is listening on.
#
# @param [String[1]] weblogic_user
#    The base WebLogic username to configure.
#
# @param [Optional[String[1]]] weblogic_password
#    The password of the base WebLogic user name.
#
# @param [Optional[String[1]]] user_config_file
#    
#
# @param [Optional[String[1]]] user_key_file
#    
#
# @param [Optional[String[1]]] userConfigFile
#    
#
# @param [Optional[String[1]]] userKeyFile
#    
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] server
#    
#
# @param [Stdlib::Absolutepath] log_dir
#    Directory to use for logging.
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Optional[Variant[Boolean, Enum['on_failure']]]] log_output
#    **DEPRECATED** This parameter is replaced by 'logoutput' and will be removed in a future release.
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#--++--
define wls_install::fmwlogdir (
  Stdlib::Absolutepath $middleware_home_dir = lookup({name => 'wls_middleware_home_dir'}),
  String[1]            $adminserver_address = lookup({name => 'domain_adminserver_address', default_value => 'localhost'}),
  Integer              $adminserver_port    = lookup({name => 'domain_adminserver_port', default_value => 7001}),
  String[1]            $weblogic_user       = lookup({name => 'wls_weblogic_user', default_value => 'weblogic'}),
  Optional[String[1]]  $weblogic_password   = lookup({name => 'domain_wls_password', default_value => undef}),
  Optional[String[1]]  $user_config_file    = lookup({name => 'domain_user_config_file', default_value => undef}),
  Optional[String[1]]  $user_key_file       = lookup({name => 'domain_user_key_file', default_value => undef}),
  Optional[String[1]]  $userConfigFile      = lookup({name => 'domain_user_config_file', default_value => undef}),
  Optional[String[1]]  $userKeyFile         = lookup({name => 'domain_user_key_file', default_value => undef}),
  String[1]            $os_user             = lookup({name => 'wls_os_user'}),
  String[1]            $os_group            = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath $download_dir        = lookup({name => 'wls_download_dir'}),
  String[1]            $server              = 'AdminServer',
  Stdlib::Absolutepath $log_dir             = lookup({name => 'wls_log_dir', default_value => undef}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                    $logoutput           = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                    $log_output          = undef, # DEPRECATED
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $userConfigFile != undef ) {
    deprecation (log_output, "Parameter 'userConfigFile' is deprecated and is replaced by 'user_config_file'. It will be removed in a future release")
    $user_config_file = $userConfigFile
  }

  if ( $userKeyFile != undef ) {
    deprecation (log_output, "Parameter 'userKeyFile' is deprecated and is replaced by 'user_key_file'. It will be removed in a future release")
    $user_config_file = $userKeyFile
  }

  $execPath = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

  # use userConfigStore for the connect
  if ( $weblogic_password ) {
    # override if config and key files are provided
    if ( $user_config_file and $user_key_file ) {
      $use_store_config = true
    }
    else {
      $use_store_config = false
    }
  } else {
    $use_store_config = true
  }

  # the py script used by the wlst
  file { "${download_dir}/${title}changeFMWLogFolder.py":
    ensure    => present,
    path      => "${download_dir}/${title}changeFMWLogFolder.py",
    content   => template('wls_install/wlst/changeFMWLogFolder.py.erb'),
    replace   => true,
    mode      => '0555',
    owner     => $os_user,
    group     => $os_group,
    backup    => false,
    show_diff => false,
  }

  exec { "execwlst ${title}changeFMWLogFolder.py":
    command   => "${middleware_home_dir}/oracle_common/common/bin/wlst.sh ${download_dir}/${title}changeFMWLogFolder.py ${weblogic_password}",
    unless    => "ls -l ${log_dir}/${server}-diagnostic.log",
    require   => File["${download_dir}/${title}changeFMWLogFolder.py"],
    path      => $execPath,
    user      => $os_user,
    group     => $os_group,
    logoutput => $logoutput,
  }
}
